import logging
import click
import yaml
import os
from typing import Dict, Any, Optional
from dotenv import load_dotenv
from google_a2a.common.types import AgentSkill, AgentCapabilities, AgentCard
from ephor_cli.server import A2AProxyServer
from ephor_cli.task_manager import AgentTaskManager
from colorama import Fore, Style
import colorama

load_dotenv()

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def load_agent_config(config_path: str) -> Dict[str, Any]:
    """Load agent configuration from a YAML file."""
    try:
        with open(config_path, "r") as file:
            return yaml.safe_load(file)
    except Exception as e:
        logger.error(f"Failed to load config from {config_path}: {e}")
        raise


def create_agent_card(
    config: Dict[str, Any], host: str, port: int
) -> tuple[AgentCard, str, Optional[Dict[str, Any]]]:
    """Create an AgentCard from a configuration dictionary."""
    agent_name = config.get("name", "Unnamed Agent")
    agent_id = agent_name.lower().replace(" ", "-")

    # Create skills
    skills = []
    for skill_config in config.get("skills", []):
        skill = AgentSkill(
            id=skill_config.get("id"),
            name=skill_config.get("name"),
            description=skill_config.get("description"),
            tags=skill_config.get("tags", []),
            examples=skill_config.get("examples", []),
            inputModes=["text"],
            outputModes=["text"],
        )
        skills.append(skill)

    # Create capabilities
    cap_config = config.get("capabilities", {})
    capabilities = AgentCapabilities(streaming=cap_config.get("streaming", False))

    # Create agent card
    agent_card = AgentCard(
        name=config.get("name", "Unnamed Agent"),
        description=config.get("description", ""),
        url=config.get("url", f"http://{host}:{port}/{agent_id}"),
        version=config.get("version", "0.1.0"),
        defaultInputModes=config.get("defaultInputModes", ["text"]),
        defaultOutputModes=config.get("defaultOutputModes", ["text"]),
        capabilities=capabilities,
        skills=skills,
    )

    # Get tool configurations if available
    tool_config = None
    if "mcpServers" in config:
        tool_config = {
            server["name"]: {
                "url": server["url"],
                "transport": server["transport"],
            }
            for server in config.get("mcpServers", [])
        }

    return agent_card, agent_id, tool_config


@click.group()
def cli():
    """Ephor CLI for managing A2A proxy agents."""
    pass


@cli.command()
@click.option("--host", default="localhost", help="Host to run the server on")
@click.option("--port", default=10002, help="Port to run the server on")
@click.option(
    "--config", "-c", multiple=True, required=True, help="Path to agent config file(s)"
)
def up(host, port, config):
    """Start A2A compatible agents loaded from config files."""
    server = A2AProxyServer(host=host, port=port)

    # Load agent configs and register them
    for config_path in config:
        try:
            logger.info(f"Loading agent config from {config_path}")
            agent_config = load_agent_config(config_path)
            agent_card, agent_id, tool_config = create_agent_card(
                agent_config, host, port
            )

            # Create task manager with tool config if available
            task_manager = AgentTaskManager(
                tool_config=tool_config, prompt=agent_config.get("prompt")
            )

            # Register the agent
            server.register_agent(
                agent_id=agent_id, agent_card=agent_card, task_manager=task_manager
            )
            logger.info(
                f"Registered agent '{agent_card.name}' with ID: {agent_id}, URL: http://{host}:{port}/{agent_id}"
            )
        except Exception as e:
            logger.error(f"Failed to register agent from {config_path}: {e}")

    logger.info(f"Starting server on http://{host}:{port}")
    server.start()


@cli.command()
@click.option(
    "--output", "-o", default=None, help="Output file path for the agent config"
)
def create_agent(output):
    """Create a new agent configuration file by answering prompts."""
    try:
        colorama.init()
    except ImportError:
        # Fallback if colorama is not installed
        class DummyFore:
            def __getattr__(self, _):
                return ""

        class DummyStyle:
            def __getattr__(self, _):
                return ""

        global Fore, Style
        Fore = DummyFore()
        Style = DummyStyle()

    click.echo(
        f"{Fore.GREEN}===== Creating New Agent Configuration ====={Style.RESET_ALL}"
    )

    # Collect basic information
    name = click.prompt(f"{Fore.CYAN}Agent name{Style.RESET_ALL}")
    description = click.prompt(f"{Fore.CYAN}Agent description{Style.RESET_ALL}")
    version = click.prompt(
        f"{Fore.CYAN}Agent version{Style.RESET_ALL}", default="1.0.0"
    )

    # Capabilities
    streaming = click.confirm(
        f"{Fore.CYAN}Enable streaming capability?{Style.RESET_ALL}", default=True
    )

    # Skills
    skills = []
    click.echo(f"{Fore.YELLOW}Now let's define the agent's skills:{Style.RESET_ALL}")

    while True:
        if skills and not click.confirm(
            f"{Fore.YELLOW}Add another skill?{Style.RESET_ALL}", default=False
        ):
            break

        skill_id = click.prompt(f"{Fore.CYAN}Skill ID{Style.RESET_ALL}")
        skill_name = click.prompt(f"{Fore.CYAN}Skill name{Style.RESET_ALL}")
        skill_description = click.prompt(
            f"{Fore.CYAN}Skill description{Style.RESET_ALL}"
        )

        # Tags
        tags = []
        click.echo(f"{Fore.BLUE}Enter tags (empty line to finish):{Style.RESET_ALL}")
        while True:
            tag = click.prompt(f"{Fore.BLUE}Tag{Style.RESET_ALL}", default="")
            if not tag:
                break
            tags.append(tag)

        # Examples
        examples = []
        click.echo(
            f"{Fore.BLUE}Enter example queries (empty line to finish):{Style.RESET_ALL}"
        )
        while True:
            example = click.prompt(f"{Fore.BLUE}Example{Style.RESET_ALL}", default="")
            if not example:
                break
            examples.append(example)

        skill = {
            "id": skill_id,
            "name": skill_name,
            "description": skill_description,
            "tags": tags,
            "examples": examples,
            "inputModes": ["text"],
            "outputModes": ["text"],
        }

        skills.append(skill)

    # Prompt
    click.echo(f"{Fore.YELLOW}Enter the system prompt for your agent:{Style.RESET_ALL}")
    prompt = click.edit(text="You are an agent. Your job is to...\n")

    # MCP Servers
    mcp_servers = []
    if click.confirm(
        f"{Fore.YELLOW}Would you like to configure MCP servers?{Style.RESET_ALL}",
        default=False,
    ):
        while True:
            server_name = click.prompt(f"{Fore.CYAN}Server name{Style.RESET_ALL}")
            server_url = click.prompt(f"{Fore.CYAN}Server URL{Style.RESET_ALL}")
            server_transport = click.prompt(
                f"{Fore.CYAN}Transport type{Style.RESET_ALL}",
                type=click.Choice(["sse", "websocket", "http"], case_sensitive=False),
                default="sse",
            )

            mcp_servers.append(
                {"name": server_name, "url": server_url, "transport": server_transport}
            )

            if not click.confirm(
                f"{Fore.YELLOW}Add another MCP server?{Style.RESET_ALL}", default=False
            ):
                break

    # Create the config dictionary
    config = {
        "name": name,
        "description": description,
        "version": version,
        "capabilities": {"streaming": streaming},
        "skills": skills,
        "prompt": prompt,
    }

    if mcp_servers:
        config["mcpServers"] = mcp_servers

    # Determine output file path
    if not output:
        output = f"agent-{name.lower().replace(' ', '-')}.yml"
        if not output.endswith(".yml"):
            output += ".yml"

    # Save to file
    output_path = os.path.abspath(output)
    with open(output_path, "w") as f:
        yaml.dump(config, f, default_flow_style=False, sort_keys=False)

    click.echo(
        f"{Fore.GREEN}Agent configuration saved to: {output_path}{Style.RESET_ALL}"
    )


def main():
    cli()


if __name__ == "__main__":
    main()
